#!/usr/bin/python
# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Tommy Brunn tommy.brunn@gmail.com
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
### END LICENSE

import os, errno, time, json, re
import pingdom
import u1db
import logging

from gi.repository import Gtk, Notify # pylint: disable=E0611
from gi.repository import AppIndicator3 as AppIndicator

from indicator_pingdom_lib.helpers import get_media_file, get_gtk_builder
from indicator_pingdom.AboutIndicatorPingdomDialog import AboutIndicatorPingdomDialog
from indicator_pingdom.pingdomaccount import PingdomAccount
from indicator_pingdom_lib.indicator_pingdomconfig import __database_dir__
from indicator_pingdom_lib.Builder import Builder
from pingdom.exception import PingdomError

import gettext
from gettext import gettext as _
gettext.textdomain('indicator-pingdom')
logger = logging.getLogger('indicator_pingdom')

class Indicator:
    def __init__(self, api_key):
        Notify.init("indicator-pingdom")
        self.indicator = AppIndicator.Indicator.new('indicator-pingdom', '', AppIndicator.IndicatorCategory.APPLICATION_STATUS)
        self.indicator.set_status(AppIndicator.IndicatorStatus.ACTIVE)
        self.api_key = api_key

        #Get the icon and set it
        #Try to use an icon that matches the theme
        defaults = Gtk.Settings.get_default()
        theme_name = defaults.get_property('gtk-icon-theme-name')
        try:
            icon_uri = get_media_file(os.path.join("icons", theme_name, "indicator-pingdom.svg"))
        except TypeError:
            #If there's no icon for that theme, fall back to the application icon
            icon_uri = get_media_file(os.path.join("icons", "indicator-pingdom.svg"))
        icon_path = icon_uri.replace("file:///", '')
        self.indicator.set_icon(icon_path)

        #Create the database dir if it doesn't exist
        try:
            os.makedirs(__database_dir__)
        except OSError, e:
            if e.errno != errno.EEXIST:
                raise

        #Access the database
        self.database = u1db.open(os.path.join(__database_dir__, 'accounts.u1db'), create=True)
        
        self.menu = Gtk.Menu()

        self.AboutDialog = AboutIndicatorPingdomDialog

        self.create_menu_options()
        #Attach the menu to the indicator
        self.indicator.set_menu(self.menu)
        self.menu.show()

        self.accounts = self.get_accounts()

        #Add checks and accounts to the menu
        self.populate_menu(self.accounts)

        #Build our windows
        self.initialize_windows()

    def get_accounts(self):
        '''Returns a list of all accounts'''
        accounts = []

        for document in self.database.get_all_docs()[1]:
            accounts.append(PingdomAccount(document.content["email"], document.content["password"], self.api_key, document.doc_id, document.content["created_at"], document.content["last_updated"]))

        return self.sort_accounts(accounts)

    def add_account(self, email, password, db_id=None, new=True):
        account = PingdomAccount(email, password, self.api_key, db_id, time.time(), time.time())

        if account.error != None:
            del account
            return False

        if new:
            #add to database
            try:
                db_id = self.database.create_doc(json.dumps({'email': email, 'password': password, 'created_at' : time.time(), 'last_updated' : time.time()}))
                account.id = db_id
            except u1db.errors.RevisionConflict:
                return False

        #Get the most recently added account
        self.accounts = self.get_accounts()
        account = self.accounts[-1]

        #Remove the static menu items
        self.remove_menu_options()

        #Add the new account
        self.add_account_to_menu(account)

        #Re-add the static menu items
        self.create_menu_options()

        #Update the accounts list in the UI
        self.accountswindow["accounts_treeiters"].append(self.accountswindow["accounts_liststore"].append([email]))
        return True

    def edit_account(self, account, email, password):
        '''Not implemented yet'''
        return False
        #Update the database entry
        try:
            document = self.database.get_doc(account.id)
            document.content = json.dumps({"email" : email, "password": password, "created_at" : document.content["created_at"], "last_updated" : time.time()})
            self.database.put_doc(document)
        except:
            pass

        self.remove_account(account)
        self.add_account(email, password, document.doc_id, new=False)

    def remove_account(self, account):
        '''Removes an account and all associated checks'''

        #Remove the account from the database
        document = self.database.get_doc(account.id)
        try:
            self.database.delete_doc(document)
        except:
            pass

        #Remove the items from the menu
        self.remove_account_from_menu(account)

        #Remove the account from our internal list
        self.accounts.remove(account)

        #Not sure if this is necessary, but it might help to clean up the timers quicker
        for check in account.checks:
            del check

    def sort_accounts(self, accounts):
        '''Returns a list sorted with the most recently updated last'''
        return sorted(accounts, key=lambda k: k.last_updated)

    def add_account_to_menu(self, account):
        #Add account heading
        account.headingItem = Gtk.MenuItem(account.email)
        account.headingItem.set_sensitive(False)
        account.headingItem.show()
        self.menu.append(account.headingItem)

        for check in account.checks:
            check.menuItem.show()
            self.menu.append(check.menuItem)

            if not check.is_up():
                check.messageItem.show()
            
            self.menu.append(check.messageItem)

        #Add a separator
        account.separator = Gtk.SeparatorMenuItem()
        account.separator.show()
        self.menu.append(account.separator)

    def remove_account_from_menu(self, account):
        self.menu.remove(account.headingItem)
        self.menu.remove(account.separator)
        for check in account.checks:
            self.menu.remove(check.menuItem)
            self.menu.remove(check.messageItem)

    def populate_menu(self, accounts):
        '''Repopulates the menu with all accounts and checks'''
        #Clear the menu
        self.remove_menu_options()

        #Add accounts and checks to menu
        for account in accounts:
            self.add_account_to_menu(account)

        #Re-add the items that should always be there
        self.create_menu_options()

    def create_menu_options(self):
        '''Adds permanent items to the menu'''
        # Add items to Menu and connect signals.

        #Adding accounts button
        self.edititem = Gtk.MenuItem(_("Edit accounts"))
        self.edititem.show()
        self.edititem.connect("activate", self.show_accounts_window)
        self.menu.append(self.edititem)
        
        #Adding about button 
        self.aboutitem = Gtk.MenuItem("About")
        self.aboutitem.connect("activate", self.on_about_activated)
        self.aboutitem.show()
        self.menu.append(self.aboutitem)

        self.quititem = Gtk.MenuItem("Quit")
        self.quititem.connect("activate", self.quit)
        self.quititem.show()
        self.menu.append(self.quititem)

    def remove_menu_options(self):
        '''Removes the "permanent" items from the menu.'''
        self.menu.remove(self.edititem)
        self.menu.remove(self.aboutitem)
        self.menu.remove(self.quititem)

    def initialize_windows(self):
        self.builder = get_gtk_builder("AccountsWindow")

        signals = {
            "on_delete_event" : self.hide_accounts_window,
            "on_accounts_selected" : self.on_accounts_selected,
            "on_remove_account_button_activate" : self.on_remove_account_clicked,
            "on_add_account_button_activate" : self.show_add_account_window,
            "on_edit_account_button_activate" : self.show_edit_account_window,
            "on_okbutton_activate" : self.apply_edit_account,
            "on_cancelbutton_activate" : self.cancel_edit_account
        }

        self.builder.connect_signals(signals)

        #Lets throw all the stuff related to the accountswindow in a dictionary
        self.accountswindow = {}

        self.accountswindow["window"] = self.builder.get_object("accountswindow")
        self.accountswindow["add_button"] = self.builder.get_object("add_account_button")
        self.accountswindow["remove_button"] = self.builder.get_object("remove_account_button")
        self.accountswindow["edit_button"] = self.builder.get_object("edit_account_button")
        
        self.editaccount = {}
        self.editaccount["window"] = self.builder.get_object("edit_account_window")
        self.editaccount["ok_button"] = self.builder.get_object("okbutton")
        self.editaccount["cancel_button"] = self.builder.get_object("cancelbutton")     
        self.editaccount["emailentry"] = self.builder.get_object("emailentry")
        self.editaccount["passwordentry"] = self.builder.get_object("passwordentry")
        self.editaccount["loginmessage"] = self.builder.get_object("loginmessage")
        self.editaccount["errormessage"] = self.builder.get_object("errormessage")

        #Theme our error message
        context = self.editaccount["errormessage"].get_style_context()
        context.add_class(Gtk.STYLE_CLASS_ERROR)

        self.accountswindow["accounts_liststore"] = self.builder.get_object("accountslist")
        self.accountswindow["accounts_treeiters"] = []

        #Load the accounts into our liststore and then into a list of treeiters
        for account in self.accounts:
            self.accountswindow["accounts_treeiters"].append(self.accountswindow["accounts_liststore"].append([account.email]))        

    def show_accounts_window(self, widget, data=None):
        self.accountswindow["window"].show_all()
        return True

    def hide_accounts_window(self, widget, data=None):
        self.accountswindow["window"].hide()
        return True

    def on_accounts_selected(self, widget):
        '''Makes edit and remove buttons sensitive when an account is selected'''
        self.accountswindow["remove_button"].set_sensitive(True)
        self.accountswindow["edit_button"].set_sensitive(True)

    def show_add_account_window(self, *widget):
        self.editaccount["emailentry"].set_text("")
        self.editaccount["passwordentry"].set_text("")
        
        self.editaccount["window"].show_all()

    def show_edit_account_window(self, widget):
        #Get the selected row
        (model, treeiter) = widget.get_selected()
        if treeiter == None:
            return True

        #Find the corresponding account
        for account in self.accounts:
            if account.email == model.get_value(treeiter, 0):
                self.editaccount["emailentry"].set_text(account.email)
                self.editaccount["passwordentry"].set_text(account.password)
                self.editaccount["window"].show_all()

        return True

    def apply_edit_account(self, widget):
        self.editaccount["ok_button"].set_sensitive(False)
        email = self.editaccount["emailentry"].get_text()
        password = self.editaccount["passwordentry"].get_text()

        for widget in self.editaccount["loginmessage"].get_children():
            widget.show()
        self.editaccount["loginmessage"].show()

        #Quick check to see if it seems like an email and real password
        if not re.match(r"[^@]+@[^@]+\.[^@]+", email) or len(password) < 5:
            for widget in self.editaccount["loginmessage"].get_children():
                widget.hide()
            self.editaccount["loginmessage"].hide()
            self.editaccount["errormessage"].show()
            return False

        #If there's already such an account, update it
        for account in self.accounts:
            if email == account.email:
                self.edit_account(account, email, password)
                self.cancel_edit_account(widget)
                return True

        #If this is a new account, add it
        if self.add_account(email, password) == False:
            for widget in self.editaccount["loginmessage"].get_children():
                widget.hide()
            self.editaccount["loginmessage"].hide()
            self.editaccount["errormessage"].show()
            self.editaccount["ok_button"].set_sensitive(True)
        else:
            logger.info("Created account {}".format(email))
            #Close the window
            self.cancel_edit_account(widget)

    def cancel_edit_account(self, *widget):
        self.editaccount["ok_button"].set_sensitive(True)
        self.editaccount["errormessage"].hide()
        for widget in self.editaccount["loginmessage"].get_children():
            widget.hide()
        self.editaccount["loginmessage"].hide()
        self.editaccount["window"].hide()

    def on_remove_account_clicked(self, widget):
        '''Removes an account from the accounts window'''

        #Get the selected row
        (model, treeiter) = widget.get_selected()
        if treeiter == None:
            return True

        #Find the corresponding account
        for account in self.accounts:
            if account.email == model.get_value(treeiter, 0):
                self.remove_account(account)
                break

        #remove the row from the liststore
        model.remove(treeiter)
        return True

    def on_about_activated(self, widget, data=None):
        """Display the about box for indicator-pingdom."""
        if self.AboutDialog is not None:
            about = self.AboutDialog() # pylint: disable=E1102
            response = about.run()
            about.destroy()

    def quit(self, *data):
        Gtk.main_quit()
    
def new_application_indicator(api_key):
    ind = Indicator(api_key)
    return ind.indicator
